==> django_restql-0.3.0-py3-none-any.whl: django_restql/mixins.py <==
import re
import json

import dictfier
from rest_framework.response import Response
from django.shortcuts import get_object_or_404

def format_query(query, schema):
    for field in query:
        if isinstance(field, dict):
            for nested_field in field:
                if isinstance(schema[nested_field], list):
                    # Iterable nested field
                    field[nested_field] = [field[nested_field]]
                    format_query(schema[nested_field], field[nested_field][0])
                else:
                    # Flat nested field
                    format_query(schema[nested_field], field[nested_field])
        else:
            pass


def parse_query(query):
    # Match field, '{', '}' and ','
    regax = r"[\{\}\,]|\w+"
    query_nodes = re.findall(regax, query)
    raw_json = []
    for i, node in enumerate(query_nodes):
        if node == "{":
            if i == 0:
                prev = '"result"'
            else:
                prev = raw_json.pop()
            raw_json.append("{" + prev + ":" + "[")
        elif node == "}":
            raw_json.append("]}")
        elif node == ",":
            raw_json.append(node)
        else:
            raw_json.append('"'+node+'"')

    json_string = "".join(raw_json)
    data = json.loads(json_string)
    return data["result"]


class DynamicFieldsMixin():
    query_param_name = "query"

    def list(self, request):
        schema = self.get_serializer().data
        queryset = self.get_queryset()
        if self.filter_backends is not None:
            queryset = self.filter_queryset(queryset)
        if self.pagination_class is not None:
            queryset = self.paginate_queryset(queryset)

        serializer = self.get_serializer(
            queryset,
            many=True,
            context={'request': request}
        )

        response = Response
        if self.paginator is not None:
            response = self.get_paginated_response

        if self.query_param_name in request.query_params:
            query = parse_query(request.query_params[self.query_param_name])

            format_query(query, schema)
            query = [query]
            data = dictfier.filter(
                serializer.data,
                query
            )

            return response(data)
        return response(serializer.data)

    def retrieve(self, request, pk=None):
        schema = self.get_serializer().data
        queryset = self.get_queryset()
        object = get_object_or_404(queryset, pk=pk)
        serializer = self.get_serializer(
            object, context={'request': request}
        )

        if self.query_param_name in request.query_params:
            query = parse_query(request.query_params[self.query_param_name])

            format_query(query, schema)
            data = dictfier.filter(
                serializer.data,
                query
            )
            return Response(data)
        return Response(serializer.data)

==> django_restql-0.3.1-py3-none-any.whl: django_restql/mixins.py <==
import re
import json

import dictfier
from rest_framework.response import Response
from django.shortcuts import get_object_or_404


def get_formatted_query(raw_query, schema):
    fields = []
    for field in raw_query:
        if isinstance(field, dict):
            for nested_field in field:
                if isinstance(schema[nested_field], list):
                    # Iterable nested field
                    nested_iterable_node = get_formatted_query(
                        field[nested_field],
                        schema[nested_field]
                    )
                    fields.append({nested_field: [nested_iterable_node]})
                else:
                    # Flat nested field
                    nested_flat_node = get_formatted_query(
                        field[nested_field],
                        schema[nested_field]
                    )
                    fields.append({nested_field: nested_flat_node})
        else:
            # Flat field
            fields.append(field)
    return fields


def parse_query(query_str):
    # Match field, '{', '}' and ','
    regax = r"[\{\}\,]|\w+"
    query_nodes = re.findall(regax, query_str)
    raw_json = []
    for i, node in enumerate(query_nodes):
        if node == "{":
            if i == 0:
                prev = '"result"'
            else:
                prev = raw_json.pop()
            raw_json.append("{" + prev + ":" + "[")
        elif node == "}":
            raw_json.append("]}")
        elif node == ",":
            raw_json.append(node)
        else:
            raw_json.append('"'+node+'"')

    json_string = "".join(raw_json)
    raw_query = json.loads(json_string)["result"]
    return raw_query


class DynamicFieldsMixin():
    query_param_name = "query"

    def list(self, request):
        schema = self.get_serializer().data
        queryset = self.get_queryset()
        if self.filter_backends is not None:
            queryset = self.filter_queryset(queryset)
        if self.pagination_class is not None:
            queryset = self.paginate_queryset(queryset)

        serializer = self.get_serializer(
            queryset,
            many=True,
            context={'request': request}
        )

        response = Response
        if self.paginator is not None:
            response = self.get_paginated_response

        if self.query_param_name in request.query_params:
            query_str = request.query_params[self.query_param_name]
            raw_query = parse_query(query_str)

            query = get_formatted_query(raw_query, schema)
            query = [query]  # extra [] because a list is returned in this case
            data = dictfier.filter(
                serializer.data,
                query
            )

            return response(data)
        return response(serializer.data)

    def retrieve(self, request, pk=None):
        schema = self.get_serializer().data
        queryset = self.get_queryset()
        object = get_object_or_404(queryset, pk=pk)
        serializer = self.get_serializer(
            object, context={'request': request}
        )

        if self.query_param_name in request.query_params:
            query_str = request.query_params[self.query_param_name]
            raw_query = parse_query(query_str)

            query = get_formatted_query(raw_query, schema)
            data = dictfier.filter(
                serializer.data,
                query
            )
            return Response(data)
        return Response(serializer.data)
